local ui = {}

function ui:load()
	self.array = {}

	self.font =  {
			small = love.graphics.newFont("data/font/FORCED_SQUARE.ttf", math.floor(settings.screen.width * 0.03)),
			large = love.graphics.newFont("data/font/FORCED_SQUARE.ttf", math.floor(settings.screen.width * 0.05)),
			huge = love.graphics.newFont("data/font/FORCED_SQUARE.ttf", math.floor(settings.screen.width * 0.08)),
			title = love.graphics.newFont("data/font/FORCED_SQUARE.ttf", math.floor(settings.screen.width * 0.16))
		}

	self.shadowColor = {0, 0, 0, 0.1}
	self.shadowDistance = settings.screen.height * 0.02
end

local function setAlpha(_a)
	local r, g, b, a = love.graphics.getColor()
	love.graphics.setColor(r, g, b, a * _a)
end

--==[ 'CREATE SHIT' TYPE FUNCTIONS ]==--
function ui:newTitle(text, x, y, hx, hy, font, _color, _align, _shadow, _id)
	_color = _color or {1, 1, 1, 1}
	_align = _align or "left"
	_shadow = _shadow or false
	_id = _id or self:countItems() + 1

	self.array[_id] = {
		hidden = false,
		type = "title",
		text = text,
		x = x,
		y = y,
		dx = x,
		dy = y,
		sx = x,
		sy = y,
		hx = hx,
		hy = hy,
		font = font,
		color = _color,
		alpha = 1,
		dalpha = 1,
		align = _align,
		shadow = _shadow,
		id = _id
	}
end

function ui:newButton(func, text, x, y, hx, hy, width, height, font, _color, _textColor, _hoverColor, _shadow, _id)
	_color = _color or {1, 1, 1, 1}
	_textColor = _textColor or {0, 0, 0, 1}
	_shadow = _shadow or true
	_id = _id or self:countItems() + 1

	self.array[_id] = {
		hidden = false,
		type = "button",
		func = func,
		text = text,
		x = x,
		y = y,
		width = width,
		height = height,
		dx = x,
		dy = y,
		sx = x,
		sy = y,
		hx = hx,
		hy = hy,
		font = font,
		color = _color,
		textColor = _textColor,
		hoverColor = _hoverColor,
		alpha = 1,
		dalpha = 1,
		shadow = _shadow,
		id = _id,
		mouseOver = false
	}
end

--==[ 'CHANGE SHIT' TYPE FUNCTIONS ]==--
function ui:show(id, _instant)
	_instant = _instant or false
	if self.array[id] then
		self.array[id].hidden = false
		self.array[id].alpha = 1
		self.array[id].x = self.array[id].sx
		self.array[id].y = self.array[id].sy

		if _instant then
			self.array[id].dx = self.array[id].sx
			self.array[id].dy = self.array[id].sy
		end

	end
end

function ui:hide(id, _instant)
	_instant = _instant or false
	if self.array[id] then
		self.array[id].hidden = true
		self.array[id].alpha = 0
		self.array[id].x = self.array[id].hx
		self.array[id].y = self.array[id].hy

		if _instant then
			self.array[id].dx = self.array[id].hx
			self.array[id].dy = self.array[id].hy
		end
	end
end

function ui:setText(id, text)
	if self.array[id] then
		self.array[id].text = text
	end
end

function ui:setPosition(id, x, y)
	if self.array[id] then
		x = x or self.array[id].x
		y = y or self.array[id].y
		self.array[id].x = x
		self.array[id].y = y
	end
end

function ui:setColor(id, r, g, b, a)
	if self.array[id] then
		self.array[id].color = {r, g, b, a}
	end
end

--==[ CALLBACK FUNCTIONS ]==--
function ui:update(dt)
	for k,v in pairs(self.array) do
		v.dx = v.dx + (v.x - v.dx) * smoof * dt
		v.dy = v.dy + (v.y - v.dy) * smoof * dt
		v.dalpha = v.dalpha + (v.alpha - v.dalpha) * smoof * dt

		if PLATFORM == "OS X" or PLATFORM == "Windows" or PLATFORM == "Linux" then
			if v.type == "button" then
				if v.hidden == false then
					local mx, my = love.mouse.getPosition()
					v.mouseOver = false
					if mx > v.x and mx < v.x + v.width and my > v.y and my < v.y + v.height then
						v.mouseOver = true
					end
				end
			end
		end
	end
end
	

function ui:draw()
	--local n = 0
	for k,v in pairs(self.array) do
		if v.type == "title" then
			self:drawTitle(v)
		elseif v.type == "button" then
			self:drawButton(v)
		end
	end
	--love.graphics.setColor(1, 1, 1, 1)
	--love.graphics.print(n, 12, 12)
end

function ui:input(x, y)
	for k,v in pairs(self.array) do
		if v.type == "button" then
			if not v.hidden then
				if x > v.x and x < v.x + v.width and y > v.y and y < v.y + v.height then
					v.func()
				end
			end
		end
	end
end

function ui:anyButton(x, y)
	local any = false
	for k,v in pairs(self.array) do
		if v.type == "button" then
			if not v.hidden then
				if x > v.x and x < v.x + v.width and y > v.y and y < v.y + v.height then
					any = true
				end
			end
		end
	end
	return any
end

function ui:drawTitle(v)
	love.graphics.setFont(self.font[v.font])
	--SHADOW
	if v.shadow then
		love.graphics.setColor(self.shadowColor)
		setAlpha(v.dalpha)
		love.graphics.printf(v.text, math.floor(v.dx), math.floor(v.dy + ((self.font[v.font]:getAscent() - self.font[v.font]:getDescent()) * 0.1) ), settings.screen.width, v.align)
	end

	--Title
	love.graphics.setColor(v.color)
	setAlpha(v.dalpha)
	love.graphics.printf(v.text, math.floor(v.dx), math.floor(v.dy), settings.screen.width, v.align)
end

function ui:drawButton(v)
	love.graphics.setFont(self.font[v.font])
	--SHAODW
	if v.shadow then
		love.graphics.setColor(self.shadowColor)
		setAlpha(v.dalpha)
	end

	love.graphics.setColor(v.color)
	setAlpha(v.dalpha)
	love.graphics.rectangle("fill", math.floor(v.dx), math.floor(v.dy), v.width, v.height)

	if v.mouseOver then
		love.graphics.setColor(v.hoverColor)
		setAlpha(v.dalpha)
		love.graphics.rectangle("line", math.floor(v.dx), math.floor(v.dy), v.width, v.height)
	end

	local h = self.font[v.font]:getAscent() - self.font[v.font]:getDescent()
	love.graphics.setColor(v.textColor)
	love.graphics.printf(v.text, math.floor(v.dx), math.floor(v.dy) + (v.height / 2) - (h / 2), v.width, "center")
end

--==[ 'GET SHIT' TYPE FUNCTIONS ]==--
function ui:countItems()
	local i = 0
	for k,v in pairs(self.array) do
		i = i + 1
	end
	return i
end

function ui:getObject(id)
	if self.array[id] then return self.array[id] else return false end
end


return ui






