function love.load()
	PLATFORM = love.system.getOS()

	ini = require("data.class.ini")
	fmath = require("data.class.fmath")

	GAME_NAME = "SMOOF"
	GAME_VERSION = "1.0"
	DEBUG_MODE = false

	love.filesystem.setIdentity(GAME_NAME)
	--love.mouse.setVisible(false)

	--Creating window
	settings = {
		screen = {
			width = math.floor(1280),
			height = math.floor(720),
			resizable = true,
			title = GAME_NAME
		},
		game = {
			highScore = 0,
			totalFlaps = 0,
			totalTime = 0
		}
	}

	if love.filesystem.getInfo("config.ini") then
		settings = ini.load("config.ini")
	end

	if not love.filesystem.getInfo("screenshots") then
		love.filesystem.createDirectory("screenshots")
	end


	--CREAING WINDOW
	love.window.setMode(settings.screen.width, settings.screen.height, {resizable = settings.screen.resizable})
	love.window.setTitle(settings.screen.title)

	color = {
		background = {0.9, 0.9, 0.9, 1},
		square = {0.2, 0.2, 0.2, 1},
		pipe = {0.9, 0.1, 0.1, 1},
		good = {0.1, 0.9, 0.2, 1},
		bad = {0.9, 0.2, 0.2},
		gold = {1, 0.6, 0, 1}
	}

	love.graphics.setBackgroundColor(color.background)

	--Loading game
	square = require("data.class.square")
	pipe = require("data.class.pipe")
	ui = require("data.class.ui")
	vig = love.graphics.newImage("data/img/vignette.png")
	loveLogo = love.graphics.newImage("data/img/love-game-icon.png")

	inputBlock = false
	inputBlockTime = 0.2
	inputBlockTick = 0

	firstRun = true

	smoof = 16

	txt = ""

	color1 = 0
	color2 = 50
	color3 = 100
	t = 0

	text = {
		start = "PRESS SPACE TO START",
		resume = "PRESS SPACE TO RESUME",
		reset = "PRESS SPACE TO RESET"
	}

	if PLATFORM == "Android" or PLATFORM == "iOS" then
		text.start = "TAP TO START"
		text.resume = "TAP TO RESUME"
		text.reset = "TAP TO RESET"
	end

	debugFont = love.graphics.newFont(12)

	reset()
end

function reset()
	square:load()
	pipe:load()
	ui:load()

	gameStarted = false
	gamePaused = false
	gameOver = false
	showCredits = false
	gameTime = 0
	gameScore = 0
	newHighScore = false
	gameSpeed = 1
	gameRate = 0.01

	flash = 0
	flashSpeed = 8

	mdmaMode = false
	mollyTime = 10
	mollyCounter = mollytime

	gravity = settings.screen.height * 2.5
	flap = gravity * 0.4
	maxVel = settings.screen.height * 2

	canvas = love.graphics.newCanvas()

	--USER INTERFACE
	--(text, x, y, hx, hy, font, _color, _align, _shadow, _id)
	--TITLE
	ui:newTitle(GAME_NAME, 0, settings.screen.height * 0.2, 0, -(settings.screen.height * 0.3), "title", color.good, "center", true, "mainTitle")
	ui:hide("mainTitle", true)
	ui:show("mainTitle")

	--Trivia
	local triviaY = settings.screen.height - ((ui.font.large:getAscent() - ui.font.large:getDescent() ) * 3) - 12

	ui:newTitle("High score: "..settings.game.highScore.."\nTotal Time: "..(math.floor(settings.game.totalTime * 100) / 100).."\nTotal flaps: "..settings.game.totalFlaps, 12, triviaY, -(settings.screen.width * 0.3), triviaY, "large", color.square, "left", true, "trivia")
	ui:hide("trivia", true)
	ui:show("trivia")

	local i = false

	if firstRun then
		i = true
		firstRun = false
	end

	--HUD
	ui:newTitle("0", 12, 12, -(settings.screen.width * 0.1), 12, "large", color.square, "left", true, "time")
	ui:newTitle("0", 0, 12, 0, -(settings.screen.height * 0.2), "huge", color.gold, "center", true, "score")
	ui:hide("time", true)
	ui:hide("score", true)

	--Info
	ui:newTitle(text.start, 0, settings.screen.height * 0.6, 0, settings.screen.height * 1.2, "large", color.good, "center", true, "instruction")
	ui:hide("instruction", true)
	ui:show("instruction")

	--GAMEOVER
	ui:newTitle("GAME OVER", 0, settings.screen.height * 0.1, settings.screen.height * 1.2, settings.screen.height * 0.1, "huge", color.bad, "center", true, "gameoverTitle")
	ui:newTitle("SCORE: 0\nTIME: 0", 0, settings.screen.height * 0.4, -(settings.screen.width * 0.8), settings.screen.height * 0.4, "large", color.bad, "center", true, "gameoverInfo")

	ui:newTitle("NEW HIGH SCORE", 0, settings.screen.height * 0.8, 0, settings.screen.height * 1.2, "large", color.good, "center", true, "newhighscore")
	ui:hide("newhighscore", i)

	ui:hide("gameoverTitle", i)
	ui:hide("gameoverInfo", i)

	--(text, x, y, hx, hy, font, _color, _align, _shadow, _id)
	--Pause
	ui:newTitle("PAUSED", 0, settings.screen.height * 0.1, settings.screen.height * 1.2, settings.screen.height * 0.1, "huge", color.good, "center", true, "pauseTitle")
	ui:hide("pauseTitle", i)

	ui:newTitle("CREDITS", 0, settings.screen.height * 0.1, settings.screen.height * 1.2, settings.screen.height * 0.1, "huge", color.good, "center", true, "creditsTitle")
	ui:hide("creditsTitle", i)

	local creditText = "FONT: 'FORCED SQUARE' BY DrawPerfect\n\n\nMade in löve"

	ui:newTitle(creditText, 0, settings.screen.height * 0.3, 0, -(settings.screen.height * 0.3), "large", color.square, "center", true, "creditsText" )
	ui:hide("creditsText")

	--Buttons
	--(text, x, y, hx, hy, width, height, font, _color, _textColor, _hoverColor, _shadow, _id)
	local bwidth = math.floor(settings.screen.width * 0.23)
	local bheight = math.floor(settings.screen.height * 0.12)
	ui:newButton(pause, "PAUSE", settings.screen.width - bwidth - 12, settings.screen.height - bheight - 12, settings.screen.width, settings.screen.height - bheight - 12, bwidth, bheight, "large", color.square, color.background, color.good, true, "pauseButton")
	ui:hide("pauseButton")

	ui:newButton(credits, "CREDITS", settings.screen.width - bwidth - 12, settings.screen.height - bheight - 12, settings.screen.width, settings.screen.height - bheight - 12, bwidth, bheight, "large", color.square, color.background, color.good, true, "creditsButton")
	ui:hide("creditsButton", true)
	ui:show("creditsButton")

	logoSize = math.min(settings.screen.width, settings.screen.height) * 0.2
	logoX = (settings.screen.width / 2) - (logoSize / 2)
	logoY = settings.screen.height * 0.6

	logoAlpha = 0
	dlogoAlpha = 0

	--Shader
	waveShader = love.graphics.newShader([[
	extern number t;
	extern vec2 screen;
	extern number str;
	vec4 effect( vec4 color, Image texture, vec2 tc, vec2 sc )
	{
		tc.x = tc.x + sin(tc.y * 10.0 + t) * (0.0051 * str) * (screen.x * 0.001);
		tc.y = tc.y + sin(tc.x * 10.0 + t) * (0.0051 * str) * (screen.y * 0.001);
		return Texel(texture, tc) * color;
	}
	]])
	waveShader:send("screen", {settings.screen.width, settings.screen.height})
	waveShader:send("str", 0)
	waveStrength = 0
	dwaveStrength = waveStrength

	square.tsize = 0
end

function credits()

	if showCredits then
		ui:show("mainTitle")
		ui:show("instruction")
		ui:show("trivia")
		ui:hide("creditsTitle")
		ui:hide("creditsText")
		ui:setText("creditsButton", "CREDITS")
		showCredits = false
		logoAlpha = 0
	else
		ui:hide("mainTitle")
		ui:hide("instruction")
		ui:hide("trivia")
		ui:show("creditsTitle")
		ui:show("creditsText")
		ui:setText("creditsButton", "BACK")
		showCredits = true
		logoAlpha = 1
	end
end

function takeMolly()
	color1 = 0
	color2 = 50
	color3 = 100
	mdmaMode = true
	mollyCounter = mollyTime
	--time = 0.6
	flash = 1
	waveStrength = 1
end

function killMolly()
	mdmaMode = false
	time = 1
	love.graphics.setBackgroundColor(color.background)
	square.tsize = square.size
	flash = 1
	waveStrength = 0
end

function start()
	if not showCredits then
		gameStarted = true
		ui:hide("mainTitle")
		ui:show("score")
		ui:show("time")
		ui:hide("trivia")
		ui:hide("instruction")
		ui:show("pauseButton")
		ui:hide("creditsButton")
		square.tsize = square.size
	end
end

function pause()
	if gameStarted and not gameOver then
		gamePaused = not gamePaused

		if gamePaused then
			--square.tsize = 0
			ui:show("pauseTitle")
			ui:setText("instruction", text.resume)
			ui:show("instruction")
			ui:hide("pauseButton")
			ui:hide("score")
			ui:hide("time")
		else
			--square.tsize = square.size
			ui:hide("pauseTitle")
			ui:hide("instruction")
			ui:show("pauseButton")
			ui:show("score")
			ui:show("time")
		end
	end
end

function gameover()
	if not square.god then
		gameOver = true
		inputBlock = true

		ui:setText("gameoverInfo", "SCORE: "..gameScore.."\nTIME: "..math.floor(gameTime * 100) / 100)

		ui:setText("instruction", text.reset)

		ui:hide("score")
		ui:hide("time")
		ui:hide("pauseButton")
		ui:show("gameoverTitle")
		ui:show("gameoverInfo")
		ui:show("instruction")

		if gameScore > settings.game.highScore then
			settings.game.highScore = gameScore
			newHighScore = true
			ui:show("newhighscore")
		end

		settings.game.totalTime = settings.game.totalTime + gameTime

		if mdmaMode then
			killMolly()
		end

		square.tsize = 0
		for i,v in ipairs(pipe.array) do
			v.y1 = 0
			v.y2 = settings.screen.height
		end
	end
end

function sprint(t)
	txt = txt..t.."\n"
end

function love.update(dt)
	if mdmaMode then dt = dt * 0.9 end
	t = t + 2 * dt
	if t > math.pi * 2 then t = 0 end

	dlogoAlpha = dlogoAlpha + (logoAlpha - dlogoAlpha) * smoof * dt

	waveShader:send("t", t)

	dwaveStrength = dwaveStrength + (waveStrength - dwaveStrength) * smoof * dt

	waveShader:send("str", dwaveStrength)

	flash = flash + (0 - flash) * (smoof * 0.1) * dt
	
	--COLOR THING
	color1 = color1 + 30 * dt
	if color1 > 255 then color1 = 0 end

	color2 = color2 + 30 * dt
	if color2 > 255 then color2 = 0 end

	color3 = color3 + 30 * dt
	if color3 > 255 then color3 = 0 end

	--TITLE ANIMATION
	if not gameStarted then
		local y = ui:getObject("mainTitle").sy + (settings.screen.height * 0.05) * math.sin(t)
		local r, g, b, a = hsl(color2, 200, 126)
		ui:setPosition("mainTitle", false, y)
		ui:setColor("mainTitle", fmath.normal(r, 0, 255), fmath.normal(g, 0, 255), fmath.normal(b, 0, 255), 1)
	end


	--MDMA
	if mdmaMode then
		if not gamePaused then
			mollyCounter = mollyCounter - dt
			if mollyCounter < 0 then
				killMolly()
			end
		end
	end

	--INPUT BLOCK
	if inputBlock then
		inputBlockTick = inputBlockTick + dt
		if inputBlockTick > inputBlockTime then
			inputBlock = false
			inputBlockTick = 0
		end
	end

	--GAME SHIT
	--if not gamePaused then
		square:update(dt)
		pipe:update(dt)	
	--end
	ui:update(dt)

	if gameStarted and not gameOver and not gamePaused then
		gameTime = gameTime + dt
		ui:setText("time", math.floor(gameTime * 100) / 100)
		gameSpeed = gameSpeed + gameRate * dt
	end
end

function love.draw()
	love.graphics.setCanvas(canvas)
	love.graphics.clear()

	if mdmaMode then
		local r, g, b, a = hsl(color1, 200, 126)	
		love.graphics.setColor(fmath.normal(r, 0, 255), fmath.normal(g, 0, 255), fmath.normal(b, 0, 255), 1)
	else
		love.graphics.setColor(color.background)
	end
	love.graphics.rectangle("fill", 0, 0, settings.screen.width, settings.screen.height)

	if mdmaMode then
		love.graphics.push()
		love.graphics.translate(settings.screen.width / 2, settings.screen.height / 2)
		love.graphics.rotate(math.sin(t) * 0.03)
		love.graphics.scale(1 + (0.1 * math.sin(t)), 1)
		love.graphics.translate(-(settings.screen.width / 2), -(settings.screen.height / 2))
	end


	square:draw()
	pipe:draw()
	if mdmaMode then love.graphics.pop() end


	love.graphics.setColor(1, 1, 1, flash)
	love.graphics.rectangle("fill", 0, 0, settings.screen.width, settings.screen.height)
	love.graphics.setCanvas()

	love.graphics.setColor(1, 1, 1, 1)
	if mdmaMode then love.graphics.setShader(waveShader) end

	love.graphics.draw(canvas)

	love.graphics.setShader()

	love.graphics.setColor(1, 1, 1, 0.2)
	love.graphics.draw(vig, 0, 0, 0, settings.screen.width / vig:getWidth(), settings.screen.height / vig:getHeight())

	ui:draw()

	love.graphics.print(txt, 12, 12)

	--if showCredits then
	local mx, my = love.mouse.getPosition()

	love.graphics.setColor(1, 1, 1, dlogoAlpha)
	love.graphics.draw(loveLogo, logoX, logoY, 0, logoSize / loveLogo:getWidth(), logoSize / loveLogo:getHeight() )
		--logo 

	if fmath.pointInRect(mx, my, logoX, logoY, logoSize, logoSize) then
		love.graphics.setColor(0.1, 0.1, 0.1, dlogoAlpha)
		love.graphics.setBlendMode("add")
		love.graphics.draw(loveLogo, logoX, logoY, 0, logoSize / loveLogo:getWidth(), logoSize / loveLogo:getHeight() )
		love.graphics.setBlendMode("alpha")
	end

	--end

	if DEBUG_MODE then drawDebug() end
end

function drawDebug()
	love.graphics.setColor(1, 0, 0, 1)

	local s = love.graphics.getStats()
	local stats = {
		{name = "GAME VERSION", val = GAME_VERSION},
		{name = "FPS", val = love.timer.getFPS()},
		{name = "PLATFORM", val = PLATFORM},
		{name = "DRAW CALLS", val = s.drawcalls},
		{name = "CANVAS SWITCHES", val = s.canvasswitches},
		{name = "TEXTURE MEMORY", val = math.floor(s.texturememory / 1024 / 1024 * 100) / 100 .." mb"},
		{name = "CANVASES", val = s.canvases},
		{name = "LOADED FONTS", val = s.fonts},
		{name = "BATCHED DRAW CALLS", val = s.drawcallsbatched}
	}
	local str = ""
	for i,v in ipairs(stats) do
		str = str..v.name..": "..v.val.."\n"
	end
	love.graphics.setFont(debugFont)
	love.graphics.printf(str, -6, 6, settings.screen.width, "right")
end

function love.quit()
	ini.save("config.ini", settings)
end

function love.resize(w, h)
	settings.screen.width = w
	settings.screen.height = h
	reset()
end

function input(x,  y)
	if not inputBlock then
		if not ui:anyButton(x, y) then
			if not gameStarted then
				start()
			end

			if gameOver then
				reset()
			end

			if gamePaused then
				pause()
			end

			square:flap()
			ui:input(x, y)
		else
			ui:input(x, y)
		end

		if showCredits then
			if fmath.pointInRect(x, y, logoX, logoY, logoSize, logoSize) then
				love.system.openURL("http://love2d.org/")
			end
		end
	end
end

function love.keypressed(key)
	if key == "escape" then love.event.push("quit") end
	if key == "space" then input(love.mouse.getX(), love.mouse.getY()) end
	if key == "q" then DEBUG_MODE = not DEBUG_MODE end
	if key == "p" then pause() end
	if key == "f1" then
		screenshot()
	end
end

function love.mousepressed(x, y, b)
	if b == 1 then
		input(x, y)
	end
end

function love.touchpressed(id, x, y, dx, dy, pressure)
	--input()
end

function screenshot()
	love.graphics.captureScreenshot("screenshots/"..os.time()..".png")
end

function hsl(h, s, l, a)
	if s<=0 then return l,l,l,a end
	h, s, l = h/256*6, s/255, l/255
	local c = (1-math.abs(2*l-1))*s
	local x = (1-math.abs(h%2-1))*c
	local m,r,g,b = (l-.5*c), 0,0,0
	if h < 1     then r,g,b = c,x,0
	elseif h < 2 then r,g,b = x,c,0
	elseif h < 3 then r,g,b = 0,c,x
	elseif h < 4 then r,g,b = 0,x,c
	elseif h < 5 then r,g,b = x,0,c
	else              r,g,b = c,0,x
	end return (r+m)*255,(g+m)*255,(b+m)*255,a
end




